from thingz import touch_n
from thingz import touch_s
from thingz import button_b
from thingz import button_a

import board
import pulseio
import time
import ssl
import socketpool
import wifi
import adafruit_minimqtt.adafruit_minimqtt as MQTT
import random
import array

# BacPro MSPC
# Introduction à la mise en œuvre des systèmes communicants
# TP n°5
# découverte de l'utilisation du protocole MQTT
# Utilisation du capteur d'humidité
# Utilisation avec un Broker local comme Mosquitto
# version du 02/01/2023

# Ne pas modifier
clientid="a"

# Renseigner les données du point d'accès Wifi
SSID="SFR_B5F8"
mot_passe_SSID="45cdmi6ygm24wdrd5nxa"
adresse_ip_broker_mosquitto="192.168.1.29"

# topic utilisé pour la publication d'un lessage au Broker
photocell_feed = "air/humidite"

### Code ###

def display_clear():
    print('[2J', end='')
    
    
class DHT11:
    __hiLevel = 51
    def __init__(self, pin, trig_wait=18000, max_pulses=81):
        self._humidity = 0
        self._temperature = 0
        self._last_called = 0
        self._pin = pin
        self._max_pulses = max_pulses
        self._trig_wait = trig_wait
        self.pulse_in = pulseio.PulseIn(self._pin, maxlen=self._max_pulses, idle_state=True)
        self.pulse_in.pause()
    
    def _get_pulses_pulseio(self):
        pulses = array.array("H")
        self.pulse_in.clear()
        self.pulse_in.resume(self._trig_wait)
        time.sleep(0.25)
        self.pulse_in.pause()
        while self.pulse_in:
            pulses.append(self.pulse_in.popleft())
        return pulses
        
    def _pulses_to_binary(self, pulses: array.array, start: int, stop: int):
        binary = 0
        hi_sig = False
        
        for bit_inx in range(start, stop):
            if hi_sig:
                bit = 0
                if pulses[bit_inx] > self.__hiLevel:
                    bit = 1
                binary = binary << 1 | bit
            hi_sig = not hi_sig
        return binary
    
    def measure(self):
        delay_between_readings = 2
        
        if self._last_called == 0 or (time.monotonic() - self._last_called) > delay_between_readings:
        
            self._last_called = time.monotonic()
            
            new_temperature = 0
            new_humidity = 0
            
            pulses = self._get_pulses_pulseio()
            if len(pulses) < 10:
                print("DHT11 - sensor not found, check wiring")
                return
            
            if len(pulses) < 80:
                print("DHT11- A full buffer was not returned. Try again.")
                return
            
            buf = array.array("B")
            for byte_start in range(0, 80, 16):
                buf.append(self._pulses_to_binary(pulses, byte_start, byte_start + 16))
                
                
            new_humidity = buf[0]
            new_temperature = buf[2]
            
            chk_sum = 0
            for b in buf[0:4]:
                chk_sum += b
                
            if chk_sum & 0xFF != buf[4]:
                print("DHT11 - bad checksum. Try again.")
                return
                
            if new_humidity < 0 or new_humidity > 100:
                print("Received unplausible data. Try again.")
            
            self._temperature = new_temperature
            self._humidity = new_humidity
            
    @property
    def temperature(self):
        self.measure()
        return self._temperature
    @property
    def humidity(self):
        self.measure()
        return self._humidity
    
def connected(client, userdata, flags, rc):
    # Utilisée lorsque que le client est connecté au Broker
    # Le client s'inscrit au topic du Broker.
    client.subscribe(photocell_feed)
    print("Connecté au Mosquitto serveur PC ")

def disconnected(client, userdata, rc):
    # Si déconnexion du client
    print("Déconnecté du broker!")

def message(client, topic, message):
    # envoi de la donnée au topic
    print("Nouveau message avec le topic {0}: {1}".format(topic, message))

def menu_gene():
    global choix_menu
    print('Menu capteur connecté')
    print('1. Scan SSID')
    print('2. Connexion SSID')
    print('3. Adresse IP')
    print('4. Valeurs humidite')
    print('5. Lancer la publication')
    print('Choix avec bouton B')
    print(''.join([str(x4) for x4 in ['Choix=', choix_menu, '']]))
    
def menu():
    global choix_menu, humidite
    if touch_n.was_touched():
        if choix_menu <= 4:
            choix_menu = (choix_menu if isinstance(choix_menu, int or float) else 0) + 1
            menu_gene()
    if touch_s.was_touched():
        if choix_menu > 1:
            choix_menu = (choix_menu if isinstance(choix_menu, int or float) else 0) + -1
            menu_gene()
    if button_b.was_pressed():
        if choix_menu == 1:
            display_clear()
            print('Liste SSID WIFI')
	    nb_ssid=0
	    for network in wifi.radio.start_scanning_networks():
               print('SSID=', network.ssid, ' RSSI=', network.rssi)
               nb_ssid +=1
            wifi.radio.stop_scanning_networks()
            print('Appuyer bouton A')
            while not(button_a.was_pressed()):
                pass
            menu_gene()
	else:
            if choix_menu == 2:
                display_clear()
                print("Connexion du")
                print("capteur d humidite")
                print("sur le réseau ...")
                try:
                    wifi.radio.connect(SSID, mot_passe_SSID)
                    print("Connecté au réseau")
                    print('Appuyer bouton A')
                    while not(button_a.was_pressed()):
                        pass
                    menu_gene()
                except:
                    print("Impossible de connecter")
                    print("au réseau!")
                    print("Revoir le paramétrage")
            else:
                if choix_menu == 3:
                    display_clear()
                    print('Adresse IP='+str(wifi.radio.ipv4_address))
                    print('Appuyer bouton A')
                    while not(button_a.was_pressed()):
                        pass
                    menu_gene()
                else:
                    if choix_menu == 4:
                        display_clear()
                        print("Visualisation donnees")
                        print("capteur humidité")
                        print('Appuyer bouton A')
                        print('pour revenir au menu')
                        time.sleep(5)
                        while not(button_a.was_pressed()):
                            humidite=dht11_19.humidity
                            print('Val capt H%='+str(humidite))
                            time.sleep(1)
                        menu_gene()              
                    else:
                        demarrage()

def demarrage():
    # Création d'un pool de sockets
    pool = socketpool.SocketPool(wifi.radio)

    # Définition du client MQTT pour inscription au Broker
    mqtt_client = MQTT.MQTT(
        broker=adresse_ip_broker_mosquitto,
        port=1883,
        username="a",
        password=clientid,
        socket_pool=pool,
        ssl_context=ssl.create_default_context(),
    )

    # Mise en place des méthodes pour la gestion du protocole MQTT
    mqtt_client.on_connect = connected
    mqtt_client.on_disconnect = disconnected
    mqtt_client.on_message = message

    # Connexion du client au Broker MQTT.
    print("Connexion au Broker")
    print("Mosquitto serveur PC...")
    
    mqtt_client.connect()

    while True:
        # Interrogation de file d'attente des messages
        mqtt_client.loop()
        # envoi du nouveau message
        humidite=dht11_19.humidity
        print("Envoi de la donnée: %d..." % humidite)
        mqtt_client.publish(photocell_feed, humidite)
        print("Envoyée!")
        time.sleep(5)
        
display_clear()
dht11_19 = DHT11(board.P19)
print("TP5 mspc")
print("Capteur humidite")
humidite=dht11_19.humidity
print(str(humidite))
time.sleep(5)
choix_menu = 1
menu_gene()
while True:
    menu()